package pl.itcrowd.youtrack.api;

import pl.itcrowd.youtrack.api.defaults.DateValues;
import pl.itcrowd.youtrack.api.defaults.Fields;
import pl.itcrowd.youtrack.api.defaults.StateValues;

import java.util.ArrayList;
import java.util.List;

public final class Filter {

    private long after;

    private List<Condition> conditions = new ArrayList<Condition>();

    private long maxResults;

    private long optionsLimit;

    private Filter()
    {

    }

    public Filter after(long after)
    {
        this.after = after;
        return this;
    }

    public Filter created(DateValues date)
    {
        return created(date.getFilterValue());
    }

    public Filter created(String date)
    {
        conditions.add(new Condition(Fields.created, date));
        return this;
    }

    public static Filter createdFilter(String date)
    {
        return new Filter().created(date);
    }

    public static Filter createdFilter(DateValues date)
    {
        return new Filter().created(date);
    }

    public Filter description(String description)
    {
        conditions.add(new Condition(Fields.description, description));
        return this;
    }

    public static Filter descriptionFilter(String description)
    {
        return new Filter().description(description);
    }

    public Filter freeText(String text)
    {
        conditions.add(new Condition(null, text));
        return this;
    }

    public Filter issueId(String issueId)
    {
        conditions.add(new Condition(Fields.issueId, issueId));
        return this;
    }

    public static Filter issueIdFilter(String issueId)
    {
        return new Filter().issueId(issueId);
    }

    public Filter maxResults(long maxResults)
    {
        this.maxResults = maxResults;
        return this;
    }

    public Filter optionsLimit(long optionsLimit)
    {
        this.optionsLimit = optionsLimit;
        return this;
    }

    public Filter project(String project)
    {
        conditions.add(new Condition(Fields.projectShortName, project));
        return this;
    }

    public static Filter projectFilter(String project)
    {
        return new Filter().project(project);
    }

    public Filter reporter(String reporter)
    {
        conditions.add(new Condition(Fields.reporterName, reporter));
        return this;
    }

    public static Filter reporterFilter(String reporter)
    {
        return new Filter().reporter(reporter);
    }

    public Filter resolved(DateValues date)

    {
        return resolved(date.getFilterValue());
    }

    public Filter resolved(String date)
    {
        conditions.add(new Condition(Fields.resolved, date));
        return this;
    }

    public static Filter resolvedFilter(String date)
    {
        return new Filter().resolved(date);
    }

    public static Filter resolvedFilter(DateValues date)
    {
        return new Filter().resolved(date);
    }

    public Filter state(StateValues... state)
    {
        final StringBuilder stringBuilder = new StringBuilder();
        for (StateValues value : state) {
            stringBuilder.append(",").append(value.getFilterValue());
        }
        return state(stringBuilder.length() > 0 ? stringBuilder.substring(1) : stringBuilder.toString());
    }

    public static Filter stateFilter(StateValues... state)
    {
        return new Filter().state(state);
    }

    public static Filter stateFilter(String state)
    {
        return new Filter().state(state);
    }

    public Filter summary(String summary)
    {
        conditions.add(new Condition(Fields.summary, summary));
        return this;
    }

    public static Filter summaryFilter(String summary)
    {
        return new Filter().summary(summary);
    }

    @Override
    public String toString()
    {
        StringBuilder builder = new StringBuilder();
        String space = " ";
        for (Condition condition : conditions) {
            builder.append(space);
            if (condition.field != null) {
                builder.append(condition.field.getCommand());
                builder.append(":");
            }
            builder.append(condition.value);
        }
        if (maxResults > 0) {
            builder.append("&max=").append(maxResults);
        }
        if (optionsLimit > 0) {
            builder.append("&optionsLimit=").append(optionsLimit);
        }
        if (after > 0) {
            builder.append("&after=").append(after);
        }
        return builder.length() > space.length() ? builder.substring(space.length()) : builder.toString();
    }

    public Filter unresolved()
    {
        conditions.add(new Condition(Fields.state, StateValues.Unresolved.getFilterValue()));
        return this;
    }

    public static Filter unresolvedFilter()
    {
        return new Filter().unresolved();
    }

    public Filter updated(DateValues date)
    {
        return updated(date.getFilterValue());
    }

    public Filter updated(String date)
    {
        conditions.add(new Condition(Fields.updated, date));
        return this;
    }

    public static Filter updatedFilter(String date)
    {
        return new Filter().updated(date);
    }

    public static Filter updatedFilter(DateValues date)
    {
        return new Filter().updated(date);
    }

    public Filter updater(String updater)
    {
        conditions.add(new Condition(Fields.updaterName, updater));
        return this;
    }

    public static Filter updaterFilter(String updater)
    {
        return new Filter().updater(updater);
    }

    private Filter state(String state)
    {
        conditions.add(new Condition(Fields.state, state));
        return this;
    }

    private class Condition {

        private Fields field;

        private String value;

        private Condition(Fields field, String value)
        {
            this.field = field;
            this.value = value;
        }
    }
}
