package pl.itcrowd.youtrack.api.rest;

import junit.framework.Assert;
import org.junit.Test;
import pl.itcrowd.youtrack.api.Filter;
import pl.itcrowd.youtrack.api.defaults.DateValues;
import pl.itcrowd.youtrack.api.defaults.StateValues;

public class FilterTest {

    @Test
    public void chaining()
    {
        final String reporterBernard = "reporter:bernard";
        final String updaterJacek = "updated by:Jacek";
        final String projectTST_QA = "project:TST_QA";
        final String resolvedSunday = "resolved date:Sunday";
        final String updatedThreeWeeksAgo = "updated:{Three weeks ago}";
        final String createdLastMonth = "created:{Last month}";
        final String stateDuplicate = "state:Duplicate";
        final String space = " ";
        Assert.assertEquals(stateDuplicate + space + createdLastMonth, Filter.stateFilter(StateValues.Duplicate).created(DateValues.LastMonth).toString());
        Assert.assertEquals(createdLastMonth + space + stateDuplicate, Filter.createdFilter(DateValues.LastMonth).state(StateValues.Duplicate).toString());
        Assert.assertEquals(createdLastMonth + space + stateDuplicate + space + "description:pikachu",
            Filter.createdFilter(DateValues.LastMonth).state(StateValues.Duplicate).description("pikachu").toString());
        Assert.assertEquals(reporterBernard + space + projectTST_QA, Filter.reporterFilter("bernard").project("TST_QA").toString());
        Assert.assertEquals(projectTST_QA + space + reporterBernard, Filter.projectFilter("TST_QA").reporter("bernard").toString());
        Assert.assertEquals(resolvedSunday + space + updatedThreeWeeksAgo, Filter.resolvedFilter("Sunday").updated(DateValues.ThreeWeeksAgo).toString());
        Assert.assertEquals(updatedThreeWeeksAgo + space + resolvedSunday, Filter.updatedFilter(DateValues.ThreeWeeksAgo).resolved("Sunday").toString());
        Assert.assertEquals(updatedThreeWeeksAgo + space + updaterJacek, Filter.updatedFilter("{Three weeks ago}").updater("Jacek").toString());
        Assert.assertEquals(updaterJacek + space + updatedThreeWeeksAgo, Filter.updaterFilter("Jacek").updated("{Three weeks ago}").toString());
    }

    @Test
    public void createdFilter()
    {
        Assert.assertEquals("created:2012-01-01", Filter.createdFilter("2012-01-01").toString());
        Assert.assertEquals("created:{Last month}", Filter.createdFilter(DateValues.LastMonth).toString());
        Assert.assertEquals("created:{This month}", Filter.createdFilter(DateValues.ThisMonth).toString());
        Assert.assertEquals("created:{Two weeks ago}", Filter.createdFilter(DateValues.TwoWeeksAgo).toString());
        Assert.assertEquals("created:{Three weeks ago}", Filter.createdFilter(DateValues.ThreeWeeksAgo).toString());
        Assert.assertEquals("created:{Last week}", Filter.createdFilter(DateValues.LastWeek).toString());
        Assert.assertEquals("created:{This week}", Filter.createdFilter(DateValues.ThisWeek).toString());
        Assert.assertEquals("created:Saturday", Filter.createdFilter(DateValues.Saturday).toString());
        Assert.assertEquals("created:Friday", Filter.createdFilter(DateValues.Friday).toString());
        Assert.assertEquals("created:Thursday", Filter.createdFilter(DateValues.Thursday).toString());
        Assert.assertEquals("created:Wednesday", Filter.createdFilter(DateValues.Wednesday).toString());
        Assert.assertEquals("created:Tuesday", Filter.createdFilter(DateValues.Tuesday).toString());
        Assert.assertEquals("created:Monday", Filter.createdFilter(DateValues.Monday).toString());
        Assert.assertEquals("created:Sunday", Filter.createdFilter(DateValues.Sunday).toString());
    }

    @Test
    public void descriptionFilter()
    {
        Assert.assertEquals("description:Wacek Ramtamtamski's", Filter.descriptionFilter("Wacek Ramtamtamski's").toString());
    }

    @Test
    public void projectFilter()
    {
        Assert.assertEquals("project:TST", Filter.projectFilter("TST").toString());
    }

    @Test
    public void reporterFilter()
    {
        Assert.assertEquals("reporter:bernard", Filter.reporterFilter("bernard").toString());
    }

    @Test
    public void resolvedFilter()
    {
        Assert.assertEquals("resolved date:2012-01-01", Filter.resolvedFilter("2012-01-01").toString());
        Assert.assertEquals("resolved date:{Last month}", Filter.resolvedFilter(DateValues.LastMonth).toString());
        Assert.assertEquals("resolved date:{This month}", Filter.resolvedFilter(DateValues.ThisMonth).toString());
        Assert.assertEquals("resolved date:{Two weeks ago}", Filter.resolvedFilter(DateValues.TwoWeeksAgo).toString());
        Assert.assertEquals("resolved date:{Three weeks ago}", Filter.resolvedFilter(DateValues.ThreeWeeksAgo).toString());
        Assert.assertEquals("resolved date:{Last week}", Filter.resolvedFilter(DateValues.LastWeek).toString());
        Assert.assertEquals("resolved date:{This week}", Filter.resolvedFilter(DateValues.ThisWeek).toString());
        Assert.assertEquals("resolved date:Saturday", Filter.resolvedFilter(DateValues.Saturday).toString());
        Assert.assertEquals("resolved date:Friday", Filter.resolvedFilter(DateValues.Friday).toString());
        Assert.assertEquals("resolved date:Thursday", Filter.resolvedFilter(DateValues.Thursday).toString());
        Assert.assertEquals("resolved date:Wednesday", Filter.resolvedFilter(DateValues.Wednesday).toString());
        Assert.assertEquals("resolved date:Tuesday", Filter.resolvedFilter(DateValues.Tuesday).toString());
        Assert.assertEquals("resolved date:Monday", Filter.resolvedFilter(DateValues.Monday).toString());
        Assert.assertEquals("resolved date:Sunday", Filter.resolvedFilter(DateValues.Sunday).toString());
    }

    @Test
    public void stateFilter()
    {
        Assert.assertEquals("state:{Can't Reproduce}", Filter.stateFilter(StateValues.CantReproduce).toString());
        Assert.assertEquals("state:Verified", Filter.stateFilter(StateValues.Verified).toString());
        Assert.assertEquals("state:Submitted", Filter.stateFilter(StateValues.Submitted).toString());
        Assert.assertEquals("state:Reopened", Filter.stateFilter(StateValues.Reopened).toString());
        Assert.assertEquals("state:{To be discussed}", Filter.stateFilter(StateValues.ToBeDiscussed).toString());
        Assert.assertEquals("state:Duplicate", Filter.stateFilter(StateValues.Duplicate).toString());
        Assert.assertEquals("state:Resolved", Filter.stateFilter(StateValues.Resolved).toString());
        Assert.assertEquals("state:Fixed", Filter.stateFilter(StateValues.Fixed).toString());
        Assert.assertEquals("state:Incomplete", Filter.stateFilter(StateValues.Incomplete).toString());
        Assert.assertEquals("state:{In Progress}", Filter.stateFilter(StateValues.InProgress).toString());
        Assert.assertEquals("state:New", Filter.stateFilter(StateValues.New).toString());
        Assert.assertEquals("state:Obsolete", Filter.stateFilter(StateValues.Obsolete).toString());
        Assert.assertEquals("state:Open", Filter.stateFilter(StateValues.Open).toString());
        Assert.assertEquals("state:{Won't fix}", Filter.stateFilter(StateValues.WontFix).toString());
        Assert.assertEquals("state:-{Won't fix}", Filter.stateFilter(StateValues.NotWontFix).toString());
        Assert.assertEquals("state:-Verified", Filter.stateFilter(StateValues.NotVerified).toString());
        Assert.assertEquals("state:-Verified,{Can't Reproduce},Obsolete,{Won't fix}",
            Filter.stateFilter(StateValues.NotVerified, StateValues.CantReproduce, StateValues.Obsolete, StateValues.WontFix).toString());
    }

    @Test
    public void updatedFilter()
    {
        Assert.assertEquals("updated:Today", Filter.updatedFilter(DateValues.Today).toString());
        Assert.assertEquals("updated:Yesterday", Filter.updatedFilter(DateValues.Yesterday).toString());
        Assert.assertEquals("updated:2012-01-01", Filter.updatedFilter("2012-01-01").toString());
        Assert.assertEquals("updated:{Last month}", Filter.updatedFilter(DateValues.LastMonth).toString());
        Assert.assertEquals("updated:{This month}", Filter.updatedFilter(DateValues.ThisMonth).toString());
        Assert.assertEquals("updated:{Two weeks ago}", Filter.updatedFilter(DateValues.TwoWeeksAgo).toString());
        Assert.assertEquals("updated:{Three weeks ago}", Filter.updatedFilter(DateValues.ThreeWeeksAgo).toString());
        Assert.assertEquals("updated:{Last week}", Filter.updatedFilter(DateValues.LastWeek).toString());
        Assert.assertEquals("updated:{This week}", Filter.updatedFilter(DateValues.ThisWeek).toString());
        Assert.assertEquals("updated:Saturday", Filter.updatedFilter(DateValues.Saturday).toString());
        Assert.assertEquals("updated:Friday", Filter.updatedFilter(DateValues.Friday).toString());
        Assert.assertEquals("updated:Thursday", Filter.updatedFilter(DateValues.Thursday).toString());
        Assert.assertEquals("updated:Wednesday", Filter.updatedFilter(DateValues.Wednesday).toString());
        Assert.assertEquals("updated:Tuesday", Filter.updatedFilter(DateValues.Tuesday).toString());
        Assert.assertEquals("updated:Monday", Filter.updatedFilter(DateValues.Monday).toString());
        Assert.assertEquals("updated:Sunday", Filter.updatedFilter(DateValues.Sunday).toString());
    }

    @Test
    public void updaterFilter()
    {
        Assert.assertEquals("updated by:bernard", Filter.updaterFilter("bernard").toString());
    }
}
